/*
 * Vocoder
 *  2012 Achim Settelmeier <vocoder@m1.sirlab.de>
 *
 */

// Includes
#include <string>
#include <iostream>
#include <stdexcept>
#include "options.h"
#include "types.h"
#include "vocoder.h"
#include "track.h"
#include "inputtrack.h"
#include "outputtrack.h"
#include "vocoder.h"
#include "engine.h"

using std::cout;
using std::cerr;
using std::endl;
using std::exception;
using std::runtime_error;

// Engine
Engine::Engine(const options_t &_options):
	options(_options)
{
	
}

// run
void Engine::run(){
	const nframes_t maxFrames=65536;
	
	try{
		InputTrack formantTrack(options.formantFile, maxFrames, options.fps);
		InputTrack carrierTrack(options.carrierFile, maxFrames, options.fps);
		OutputTrack outputTrack(options.outputFile, maxFrames, options.fps);
		Vocoder vocoderLeft;
		Vocoder vocoderRight;
		vocoderLeft.setSampleRate(options.fps);
		vocoderRight.setSampleRate(options.fps);
		for(unsigned int i=0; i<VOCODER_MAXBANDS; i++){
			vocoderLeft.setBandVolume(i, options.volume, options.volume);
			vocoderRight.setBandVolume(i, options.volume, options.volume);
		}
		
		
		while(1){
			nframes_t numFramesFormant=formantTrack.readBlock(maxFrames);
			if(numFramesFormant==0)
				break;
			
			nframes_t numFramesCarrier=carrierTrack.readBlock(numFramesFormant);
			if(numFramesCarrier==0)
				break;
			nframes_t numFramesToProcess=numFramesCarrier;
			
			
			cout << "processing " << numFramesCarrier << " frames" << endl;
			processBlock(formantTrack,
				     carrierTrack,
				     outputTrack,
				     vocoderLeft,
				     vocoderRight,
				     numFramesToProcess);
			
			nframes_t numFramesWritten=outputTrack.writeBlock(numFramesToProcess);
			
			if(numFramesWritten!=numFramesToProcess)
				throw runtime_error(options.outputFile + ": write error");
			
			// no full block processed? then we're done here
			if(numFramesToProcess<maxFrames)
				break;
		}
		
		
	}catch(exception &e){
		cerr << e.what() << endl;
	}
}

void Engine::processBlock(InputTrack &formantTrack,
			  InputTrack &carrierTrack,
			  OutputTrack &outputTrack,
			  Vocoder &vocoderLeft,
			  Vocoder &vocoderRight,
			  nframes_t numFramesToProcess){
	
	sample_t *formantBufferPtr=formantTrack.getBufferPtr();
	sample_t *carrierBufferPtr=carrierTrack.getBufferPtr();
	sample_t *outputBufferPtr=outputTrack.getBufferPtr();
	
	for(nframes_t framePos=0; framePos<numFramesToProcess; framePos++){
		sample_t dummySample;
		vocoderLeft.process(formantBufferPtr + (framePos * 2),
				carrierBufferPtr + (framePos * 2),
				outputBufferPtr + (framePos * 2),
				&dummySample,
				1);
		vocoderRight.process(formantBufferPtr + (framePos * 2) + 1,
				     carrierBufferPtr + (framePos * 2) + 1,
				     &dummySample,
				     outputBufferPtr + (framePos * 2) + 1,
				     1);
	}

}
