/*
 * Vocoder
 *  2007-2012 Achim Settelmeier <vocoder@m1.sirlab.de>
 *
 * vocoder core routine
 */

#ifndef __VOCODER_H__
#define __VOCODER_H__

#include "types.h"

#define VOCODER_MAXBANDS 16


struct bandpass_t{
	sample_t low1, low2;
	sample_t mid1, mid2;
	sample_t high1, high2;
	sample_t y;
};

struct band_t{
	double volumeLeft, volumeRight;
	
	double freq;
	double c;
	double f;
	double att;
	
	double oldval;
	double decay;
	
	bandpass_t formant;
	bandpass_t carrier;
};

class Vocoder{
public:
	Vocoder();
	void process(sample_t *formant, 
		     sample_t *carrier,
		     sample_t *outLeft, 
		     sample_t *outRight,
		     const nframes_t nframes);
	
	void setSampleRate(const nframes_t sampleRate);
	void setBandVolume(const unsigned int bandNr, 
			   const double volumeLeft, 
			   const double volumeRight);
	
private:
	void init();
	static void doBandpass(band_t *b, bandpass_t *bp, sample_t sample);
	
	band_t bands[VOCODER_MAXBANDS];
	nframes_t sampleRate;
	double volumeMain;
};


#endif
